-- Copyright 2012 by Till Tantau
--
-- This file may be distributed an/or modified
--
-- 1. under the LaTeX Project Public License and/or
-- 2. under the GNU Public License
--
-- See the file doc/generic/pgf/licenses/LICENSE for more information

-- @release $Header$



---
-- This package presents some examples of how different aspects of the
-- graph drawing engine can be used. In particular, the algorithms of
-- this package are not really meant to be used to layout graphs
-- (although they can be used, in principle); rather you are invited
-- to have a look at their implementation and to adapt them to your needs.
--
-- @library

local examples


-- Load algorithms from:
require "pgf.gd.examples.SimpleDemo"
require "pgf.gd.examples.SimpleEdgeDemo"
require "pgf.gd.examples.SimpleHuffman"

