-- Copyright 2015 by Till Tantau
--
-- This file may be distributed an/or modified
--
-- 1. under the LaTeX Project Public License and/or
-- 2. under the GNU Public License
--
-- See the file doc/generic/pgf/licenses/LICENSE for more information

-- @release $Header$



---
-- A pedigree depicts parent--mate--sibling relationships between individuals.
--
-- @library

local pedigrees -- Library name

require "pgf.gd.pedigrees.Koerner2015"
