% cachepic option for Preview package 
% implements log output of cachpic specific info 
\ifPreview\else\expandafter\endinput\fi
%\pr@loadcfg{prcachepic}
\g@addto@macro\pr@ship@end{
  \ifx\cachepic@name\@undefined
  \else
    \typeout{Preview: CachePic
             \number\pr@snippet\space
             \number\wd\pr@box\space
             \number\ht\pr@box\space
             \number\dp\pr@box\space
             \cachepic@name}
    \global\let\cachepic@name\@undefined
  \fi
 }
\PreviewMacro[!!]{\cachepic}
%\PreviewMacro[!]{\cacheinput}% defined through \cachepic
\PreviewEnvironment[!]{cachepicture}
\endinput
