%%
%% This is file `p2e-luatex.def',
%% generated with the docstrip utility.
%%
%% The original source files were:
%%
%% p2e-drivers.dtx  (with options: `luatex')
%% 
%% Copyright (C) 2003-2016
%% Rolf Niepraschk, Rolf.Niepraschk@gmx.de
%% Hubert Gaesslein,
%% Josef Tkadlec, j.tkadlec@email.cz
%% 
%% This work may be distributed and/or modified under the
%% conditions of the LaTeX Project Public License, either version 1.3
%% of this license or (at your option) any later version.
%% The latest version of this license is in
%%   http://www.latex-project.org/lppl.txt
%% and version 1.3 or later is part of all distributions of LaTeX
%% version 2003/12/01 or later.
%% 
%% This work has the LPPL maintenance status "author-maintained".
%% 
%% This work consists of all files listed in `manifest.txt'.
%% 
\ProvidesFile{p2e-luatex.def}
        [2016/02/05 v0.1u
                          Driver-dependant file (RN,HjG,JT)]
\begingroup
  \ifnum\outputmode>0\relax
    \gdef\pIIe@mode{2}
  \fi
\endgroup
\ifcase\pIIe@mode\relax \or\or
  \def\pIIe@code#1{\pdfextension literal{ q #1 Q }}
  \protected\edef\pIIe@pdfliteral{\pdfextension literal}
\fi
\endinput
%%
%% End of file `p2e-luatex.def'.
