\typeout{%
!!!!!!!!!!!!!!!!!!!! WARNING !!!!!!!!!!!!!!!!!!!!^^J%
Loading the file 'bxdpxp2e.def' in 'pict2e.cfg' should be^^J%
no longer needed, because the recent pict2e package (>=0.2y)^^J%
properly supports dvipdfmx.^^J%
If your pict2e is of older version, then update pict2e, or^^J%
load 'bxeepic' package (with 'dvipdfmx' driver option) in^^J%
place of 'pict2e' package.^^J%
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
\endinput
